#!-PERL-
# fm2html.pl
#
# THIS FILE IS COPYRIGHT 1994 BY TELENOR RESEARCH, NORWAY
#
#                    ALL RIGHTS RESERVED.
#
# For rights granted to the public see the COPYRIGHT file
#
# Programming : Jon Stephenson von Tetzchner

# This script is part of the fm2html package.

$\ = "\n";		# set output record separator
$, = " ";		# set output field separator
$* = 0;			# set single line pattern matching

select( STDERR );	# all error messages go to stderr
$| = 1;			# make stderr unbuffered

$FALSE = 0;
$TRUE  = 1;

$FM_MAGIC	= "<MakerFile";
$FM_BOOK_MAGIC	= "<Bookfile";
$MIF_MAGIC	= "<MIFFile";
$MIF_BOOK_MAGIC	= "<Book ";

$EXTENSION      = "htags";


sub escape	# take string, insert backslash before special characters
{
  local( $_ ) = @_;
  s|([\.\+\?\*\'\"\(\)])|\\$1|g;
  $_;
}

sub dirname	# take pathname, return leading directories
{
  local( $_ ) = @_;
  s|^(.+)/.*$|$1| || s|^/[^/]*$|/| || s|^[^/]*$|.|;
  $_;
}

sub basename	# take pathname, strip leading directories and optional suffix
{
  local( $_, $suffix ) = @_;
  m|/|o && s|^.*/([^/]*)$|$1|;
  $suffix = &escape( $suffix );
  s|(.*)$suffix|$1|;
  $_;
}


local( $prog ) = &basename( $0 );
local( $usage ) = "Usage: $prog [-t title] [-a author] [-h header] [-x xdpi] [-y ydpi] [-ng] [-nc] [-nf] [-nt] [-s] [-q] filename";

local( $title, $author, $strip, $infile, $indir, $cwd, $base, $qflag );
local( $notoc ) = $FALSE;
local( $quiet ) = $FALSE;
local( $debug ) = $FALSE;
local( $header ) = "-HEADER-";

# *** Get command line parameters
while ( @ARGV )
{
  $param = shift;

  if ( "$param" =~ m|^-t$| )
  {
    $title = shift;
  }
  elsif ( "$param" =~ m|^-a$| )
  {
    $author = shift;
  }
  elsif ( "$param" =~ m|^-h$| )
  {
    $header = shift;
  }
  elsif ( "$param" =~ m|^-x$| )
  {
    $ENV{'XDPI'} = shift;
  }
  elsif ( "$param" =~ m|^-y$| )
  {
    $ENV{'YDPI'} = shift;
  }
  elsif ( "$param" =~ m|^-nc$| )
  {
    $ENV{'NOCROP'} = 1;
  }
  elsif ( "$param" =~ m|^-nf$| )
  {
    $ENV{'NOFRAMEFILES'} = 1;
  }
  elsif ( "$param" =~ m|^-ng$| )
  {
    $ENV{'NOGRAPHICS'} = 1;
  }
  elsif ( "$param" =~ m|^-nt$| )
  {
    $notoc = $TRUE;
    $ENV{'NOTOC'} = 1;
  }
  elsif ( "$param" =~ m|^-s$| )
  {
    $strip = "-s";
    $ENV{'STRIP'} = 1;
  }
  elsif ( "$param" =~ m|^-q$| )
  {
    $quiet = $TRUE;
    $qflag = "-q";
    $ENV{'QUIET'} = 1;
  }
  elsif ( "$param" =~ m|^-d$| )
  {
    $debug = $TRUE;
    $dflag = "-d";
    $ENV{'DEBUG'} = 1;
  }
  elsif ( "$param" =~ m|^-| )
  {
    print "$prog: ERROR: Invalid flag: \"$param\"";
    print "$usage";
    exit( 1 );
  }
  elsif ( "$infile" )
  {
    print "$prog: ERROR: only one input file may be specified";
    print "$usage";
    exit( 1 );
  }
  else
  {
    $infile = "$param";
  }
}

if ( $quiet == $FALSE )
{
  "$infile" && print "$prog: Input File: $infile";
  "$title"  && print "$prog: Title: $title";
  "$author" && print "$prog: Author: $author";
  "$header" && print "$prog: Header: $header";
}

if ( ! $infile )
{
  print "";
  print "$usage";
  print "";
  print "\tThe \"-t\" option specifies the title of the document/book";
  print "\tThe \"-a\" option specifies the author of the document/book";
  print "\tThe \"-h\" option specifies header text to go at the top of each document";
  print "\tThe \"-x\" option specifies horizontal graphics resolution";
  print "\tThe \"-y\" option specifies vertical graphics resolution";
  print "\tThe \"-ng\" flag indicates that no graphics be converted";
  print "\tThe \"-nc\" flag indicates that no cropping of graphics be done";
  print "\tThe \"-nf\" flag indicates that no FrameMaker-generated files be converted";
  print "\tThe \"-nt\" flag indicates that no table of contents be generated";
  print "\tThe \"-s\" flag indicates that \".doc\" extensions be stripped";
  print "\tThe \"-q\" flag indicates that only error messages be printed";
  print "\tThe \"-d\" flag indicates that debugging information be printed";
  print "\tThe \"filename\" parameter is the name of the document/book to translate";
  print "";
  exit( 1 );
}

if ( ! -e "$infile" )
  { die "$prog: Input file \"$infile\" does not exist... exiting"; }

open( INFILE, "<$infile" ) ||
  die "$prog: Could not open file \"$infile\"... exiting";
$idline = <INFILE>;
close( INFILE );

# Docs and xrefs may have relative pathnames so it is necessary to chdir.
# Unfortunately, however, FrameMaker sometimes has problems figuring out
# what the current directory is and defaults to the user's home directory,
# in which case it won't find the files anyway.
$indir = &dirname( "$infile" );
chdir( "$indir" ) ||
  die "mif2mif: ERROR: Can't change directory to \"$indir\"... exiting";
chop( $cwd = `pwd` );
$infile = &basename( "$infile" );
$infile = "$cwd/$infile";

if ( $idline =~ m|^$FM_MAGIC|i || $idline =~ m|^$FM_BOOK_MAGIC|i )
{	
  if ( $strip && $infile =~ m|\.doc$| )
  {
    $base = "$infile";
    $base =~ s|\.doc$||;
    $mif = "$base.mif";
  }
  else
  {
    $base = "$infile";
    $mif = "$infile.mif";
  }
  
  $quiet == $FALSE && print "$prog: Executing \"fm2mif $strip $infile\"";
  eval system( "-SCRDIR-/fm2mif $qflag $strip $infile" );

  &mif2html( $mif, $title, $author, $header );
}
elsif ( $idline =~ m|^$MIF_MAGIC|i || $idline =~ m|^$MIF_BOOK_MAGIC|i )
{ 
  &mif2html( $infile, $title, $author, $header );
}

$quiet == $FALSE && print "\n$prog: Done\n";

exit( 0 );


# *** mif2html 

sub mif2html
{
  local( $infile, $title, $author, $header ) = @_; 

  -e "$infile" ||
    die "$prog: mif2html: File \"$infile\" does not exist\n... exiting";
  open( INFILE, "<$infile" ) ||
    die "$prog: mif2html: Could not open file \"$infile\"... exiting\n";
  $idline = <INFILE>;
  close( INFILE );
  
  if ( $infile =~ m|\.mif| )
  {
    $mif = "$infile";
    $base = "$mif";
    $base =~ s|\.mif$||;
  }
  else
  {
    $base = "$infile";
    $mif = "$infile.mif";
  }
  
  $tagsfile = "$base.$EXTENSION";

  if ( -e "$tagsfile" && -f "$tagsfile" )
  {
    $quiet == $FALSE &&
      print "$prog: Using existing tags file \"$tagsfile\"";
  }
  else
  { 
    if ( $ENV{'MIF2HTMLTAGSFILE'} )
      { $defaulttagsfile = $ENV{'MIF2HTMLTAGSFILE'}; }
    else
      { $defaulttagsfile = "-LIBDIR-/htags"; }
    
    $quiet == $FALSE && print
      "$prog: Copying default tags file \"$defaulttagsfile\" to \"$tagsfile\"";
    eval system( "cp $defaulttagsfile $tagsfile" );
  }
  
  if ( ! -e $tagsfile || ! -f "$tagsfile" )
    { die "$prog: Tags file \"$tagsfile\" either does not exist or is not a file... exiting."; }
  
  $ENV{'TAGSFilename'} = "$tagsfile";
  
  if ( $idline =~ m|$MIF_MAGIC|i)
    { &makerfile( $base, $title, $author, $header ); }
  elsif ( $idline =~ m|$MIF_BOOK_MAGIC|i )
    { &bookfile( $base, $title, $author, $header ); }
}


# *** makerfile - Handles a MIF file.

sub makerfile
{
  local( $infile, $title, $author, $header ) = @_;
  
  $language = $ENV{'DOCUMENTLANGUAGE'};
  if ( ! "$language" ) 
  { 
    $language = "US"; 
    $ENV{'DOCUMENTLANGUAGE'} = "US";
  } 
  
  if ( $quiet == $FALSE )
  {
    print "$prog: Starting translation of $infile.mif to $infile.html";
    print "$prog: Language: $language";
  }

  -l "$cwd/fm2html-table.gif" && eval system( "rm -f $cwd/fm2html-table.gif" );
  eval system( "ln -s -LIBDIR-/fm2html-table.gif $cwd" );

  open( TITLEFILE, ">$infile.tmp.000T" ) ||
    die "$prog: Could not open title file \"$infile.tmp.000T\"... exiting";

  "$title"  && print TITLEFILE "<TITLE>$title</TITLE>\n";
  "$header" && print TITLEFILE "<I>$header</I>\n<HR>";
  "$title"  && print TITLEFILE "<H1>$title</H1>";
  "$author" && print TITLEFILE "<ADDRESS>$author</ADDRESS>";

  ("$title" || "$author") && print TITLEFILE "<HR>";

  close( TITLEFILE );
  
  if ( $notoc == $FALSE )
  {
    open( INDEXFILE, ">$infile.tmp.000I" ) ||
      die "$prog: Could not open index file \"$infile.tmp.000I\"... exiting";
    if ( "$language" eq "NO" )
      { print INDEXFILE "<H2>Innhold</H2>"; }
    elsif ("$language" eq "D")
      { print INDEXFILE "<H2>Inhaltsverzeichnis</H2>"; }
    else
      { print INDEXFILE "<H2>Table of Contents</H2>"; }    
    close( INDEXFILE );
  }
  
  $debug == $FALSE && eval system( "rm -f $infile.tmp.000HT" );
  eval system( "-SCRDIR-/f2htranslate $qflag $dflag $infile $infile.tmp.000HT $infile.tmp.000T $infile.tmp.000I \"$title\"" );
  
  open( INDEXFILE, ">>$infile.tmp.000I" ) ||
    die "$prog: Could not reopen index file \"$infile.tmp.000I\"... exiting";
  $notoc == $FALSE && print INDEXFILE "<HR>";
  close( INDEXFILE );
  
  eval system( "cat $infile.tmp.000T $infile.tmp.000I | -BINDIR-/insertcharacters > $infile.html" );
  eval system( "cat $infile.tmp.000HT >> $infile.html" );
  $debug == $FALSE && 
    eval system( "rm -f $infile.tmp.000T $infile.tmp.000I $infile.tmp.000HT" );
}


# *** bookfile - Handles a MIF book.

sub bookfile 
{
  local( $infile, $title, $author, $header ) = @_;
  
  $language = $ENV{'DOCUMENTLANGUAGE'};
  "$language" eq "" && ($language = "US");
  
  $ENV{'BOOKFilename'} = "$infile";
  $ENV{'BOOKTitle'} = "$title";
  
  if ( $quiet == $FALSE )
  {
    print "$prog: Starting translation of FrameMaker book file $infile.mif (and all documents in it) to $infile.html";
    print "$prog: Language: $language";
  }

  -l "$cwd/fm2html-toc.gif" && eval system( "rm -f $cwd/fm2html-toc.gif" );
  eval system( "ln -s -LIBDIR-/fm2html-toc.gif $cwd" );
  -l "$cwd/fm2html-next.gif" && eval system( "rm -f $cwd/fm2html-next.gif" );
  eval system( "ln -s -LIBDIR-/fm2html-next.gif $cwd" );
  -l "$cwd/fm2html-previous.gif" &&
    eval system( "rm -f $cwd/fm2html-previous.gif" );
  eval system( "ln -s -LIBDIR-/fm2html-previous.gif $cwd" );
  -l "$cwd/fm2html-table.gif" && eval system( "rm -f $cwd/fm2html-table.gif" );
  eval system( "ln -s -LIBDIR-/fm2html-table.gif $cwd" );
  
  open( TITLEFILE, ">$infile.tmp.000T" ) ||
    die "$prog: Could not open title file \"$infile.tmp.000T\"... exiting";

  print TITLEFILE "<!-- This file was created with the fm2html filter.";
  print TITLEFILE "The filter is copyright Norwegian Telecom Research and";
  print TITLEFILE "was programmed by Jon Stephenson von Tetzchner. -->\n";

  "$title"  && print TITLEFILE "<TITLE>$title</TITLE>\n";
  "$header" && print TITLEFILE "<I>$header</I><HR>";
  "$title"  && print TITLEFILE "<H1>$title</H1>";
  "$author" && print TITLEFILE "<ADDRESS>$author</ADDRESS>";
  ("$title" || "$author") && print TITLEFILE "<HR>";
  print TITLEFILE "";

  close( TITLEFILE );
  
  if ( $notoc == $FALSE )
  {
    open( INDEXFILE, ">$infile.tmp.000I" ) ||
      die "$prog: Could not open index file \"$infile.tmp.000I\"... exiting";
    if ( "$language" eq "NO" )
      { print INDEXFILE "<H2>Innhold</H2>"; }
    elsif ("$language" eq "D")
      { print INDEXFILE "<H2>Inhaltsverzeichnis</H2>"; }
    else
      { print INDEXFILE "<H2>Table of Contents</H2>"; }    
    close( INDEXFILE );
  }
  
  if ( -e "$infile.mif" )
    { eval system( "-BINDIR-/book < $infile.mif" ); }
  else
    { eval system( "-BINDIR-/book < $infile" ); }
  
  if ( $notoc == $FALSE )
  {
    open( INDEXFILE, ">>$infile.tmp.000I" ) ||
      die "$prog: Could not reopen index file \"$infile.tmp.000I\"... exiting";
    $notoc == $FALSE && print INDEXFILE "<HR>";
    close( INDEXFILE );
  }
  
  eval system( "cat $infile.tmp.000T $infile.tmp.000I | -BINDIR-/insertcharacters >$infile.html" );
  $debug == $FALSE && eval system( "rm $infile.tmp.000T $infile.tmp.000I" );
}
