/*
// THIS FILE IS COPYRIGHT 1994 BY TELENOR RESEARCH, NORWAY
//
//                    ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
*/

/* Programming : Jon Stephenson von Tetzchner */

#ifndef TRANS_H
#define TRANS_H


struct FMFont
{
    char *tag;
    char *weight;
    char *angle;
    int underline;	/* Boolean */
    int subscript;	/* Boolean */
    int superscript;	/* Boolean */
    struct FMFont *next_font;
};


struct FMPgf
{
    char *tag;
    struct FMFont *font;
    char alignment;	/* 'C' == Center, 'R' == Right, otherwise 'L' */
    int topseparator;	/* Boolean */
    int botseparator;	/* Boolean */
    char *pgfnumfontstr;
    struct FMFont *pgfnumfont;
    struct FMPgf *next_pgf;
};


struct FMImportObject
{
    int textrectid;
    char *importobfile;
    char *importobfiledi;
    char *brect;
    char *angle;
    char *fliplr;
    struct FMImportObject *next_importobject;
};


struct FMCellLine
{
    char *fmline;
    struct FMCellLine *next_line;
};


struct FMCell
{
    int num_lines;
    int cell_columns;	/* number of columns this cell straddles */
    char alignment;	/* 'C' == Center, 'R' == Right, otherwise 'L' */
    struct FMCellLine *line;
    struct FMCell *next_cell;
};


struct FMRow
{
    int max_num_lines;
    struct FMCell *cell;
    struct FMRow *next_row;
};


struct FMTable
{
    int id;
    int num_columns;
    int *column_widths;
    char *table_title;
    struct FMRow *header;
    struct FMRow *footer;
    struct FMRow *row;
    struct FMTable *next_table;
    int table_ref;
    char header_end[31];
    char body_end[31];
    char footer_end[31];
    char *footnote[20];
    int footnote_def_count;
};


#if defined(__STDC__)

void PrintHeaderTypeEnd(void);
int HeaderType(void);
void StartPrint(void);
void EndPrint(void);
void PageStart(void);
void PageType(void);
void ImportObjectStart(void);
void ImportObFile(void);
void ImportObFileDI(void);
void BRect(void);
void Angle(void);
void FlipLR(void);
void ImportObjectEnd(void);
void PageEnd(void);
void InsertFootnote(void);
void PrintFootnotes(void);
void NewFootnoteStart(void);
void NewFootnoteEnd(void);
void InsertReferenceStart(void);
void InsertReferenceEnd(void);
int PrintReferences(void);
int ReferenceType(void);
void AFramesStart(void);
void AFramesEnd(void);
void AFrameUse(void);
int TableAFrameUse(void);
char *GetContents(void);
char *GetValue(void);
void  SetReferenceNumber(void);
void SetReferenceFile(void);
void  SetMarker(void);
int GetNumber(void);
void ParaStart(void);
void PgfStart(void);
void ParagraphNumberFont(void);
void ParagraphNumberString(void);
void PgfEnd(void);
void ParaEnd(void);
void SetMarkerType(void);
void PgfCatalogStart(void);
void PgfCatalogEnd(void);
void FontCatalogStart(void);
void FontCatalogEnd(void);

void TableUse(void);
void GetTableID(void);
void GetTableNumColumns(void);
void TableStart(void);
void TableEnd(void);
void TablesStart(void);
void TablesEnd(void);
void TableTitleStart(void);
void TableTitleEnd(void);
int GetID(void);
void InsertTable(FILE *f, int id);
int PrintTableString(void);
int PrintTable(void);
void PrintTableRow(FILE *f, struct FMRow *row, int *column_widths);
void PrintTableCell(FILE *f, struct FMCell *cell, int line_nr, int width);
void PrintTableFootnotes(FILE *f, struct FMTable *table);
struct FMTable *FindTable(int id);
int GetFrameType(int);
void StripSpace(char *);
char *GetTextOnly(char *);
void PrintFrameItem(void);
void EchoTable(void);
int IsIndent(int tag);
int IsBullet(int tag);
int IsMark(int tag);
int IsNumber(int tag);
int IsList(int tag);    
int IsContinue(int tag);
int Level(int tag);
int ListType(int tag);
void TCDescend(int tclevels);
void TCAscend(int tclevels);
void Descend(int list_type, int levels);
void Ascend(int levels);
void PrintString(void);
void ParagraphTag(void);
void ParagraphAlignment(void);
void ParagraphTopSeparator(void);
void ParagraphBottomSeparator(void);
void FontTag(void);
void PrintHeaderTypeStart(void);
void UseString(char *string_contents);
void GetReferenceFile(void);
void FrameStart(void);
void FrameEnd(void);
int GetIDNextLine(void);
void PrintTextRectID(void);
void FontStart(void);
void FontEnd(void);
void ResetFont(void);
void FontFamily(void);
void InsertSpecialChar(void);
void TextRectStart(void);
void TextRectEnd(void);
void InsertFigure(FILE *f, int idf);
void InsertInlineFigure(FILE *f, int idf);
void SetFrameType(void);
void FontWeight(void);
void ChangeFontWeight(char *value);
void FontAngle(void);
void ChangeFontAngle(char *value);
void FontUnderline(void);
void ChangeFontUnderline(int is_underline);
void FontSubscript(void);
void ChangeFontSubscript(int is_underline);
void FontSuperscript(void);
void ChangeFontSuperscript(int is_underline);
void TextFlowStart(void);
void TextFlowTag(void);
void TextFlowEnd(void);
void InlineFrameStart(void);
void InlineFrameEnd(void);
void TableHeadStart(void);
void TableHeadEnd(void);
void TableBodyStart(void);
void TableBodyEnd(void);
void TableFootStart(void);
void TableFootEnd(void);
void TableRowStart(void);
void TableRowEnd(void);
void TableCellStart(void);
void TableCellColumns(void);
void TableCellEnd(void);
void NewCellLine(void);
int DoesTitleExist(char *);
void HandleParaLine(void);
void MergeMarkerWithString(char *, char *, char *);
void Finish(void);

#else

void PrintHeaderTypeStart();
void PrintHeaderTypeEnd();
int HeaderType();
void StartPrint();
void EndPrint();
void PageStart();
void PageType();
void ImportObjectStart();
void ImportObFile();
void ImportObFileDI();
void BRect();
void Angle();
void FlipLR();
void ImportObjectEnd();
void PageEnd();
void InsertFootnote();
void PrintFootnotes();
void NewFootnoteStart();
void NewFootnoteEnd();
void InsertReferenceStart();
void InsertReferenceEnd();
int PrintReferences();
int ReferenceType();
void AFramesStart();
void AFramesEnd();
void AFrameUse();
int TableAFrameUse();
char *GetContents();
char *GetValue();
void SetReferenceNumber();
void SetReferenceFile();
void SetMarker();
int GetNumber();
void ParaStart();
void PgfStart();
void ParagraphNumberFont();
void ParagraphNumberString();
void PgfEnd();
void ParaEnd();
void SetMarkerType();
void PgfCatalogStart();
void PgfCatalogEnd();
void FontCatalogStart();
void FontCatalogEnd();

void TableUse();
void GetTableID();
void GetTableNumColumns();
void TableStart();
void TableEnd();
void TablesStart();
void TablesEnd();
void TableTitleStart();
void TableTitleEnd();
int GetID();
void InsertTable();
void PrintTableString();
void PrintTable();
void PrintTableRow();
void PrintTableCell();
void PrintTableFootnotes();
struct FMTable *FindTable();
int GetFrameType();
void StripSpace();
char *GetTextOnly();
void PrintFrameItem();
int IsIndent();
int IsBullet();
int IsMark();
int IsNumber();
int IsList();
int IsContinue();    
int Level();
int ListType();
void TCDescend();
void TCAscend();
void Descend();    
void Ascend();
void PrintString();
void ParagraphTag();
void ParagraphAlignment();
void ParagraphTopSeparator();
void ParagraphBottomSeparator();
void FontTag();
void PrintHeaderTypeStart();
void UseString();
void GetReferenceFile();
void FrameStart();
void FrameEnd();
int GetIDNextLine();
void PrintTextRectID();
void FontStart();
void FontEnd();
void ResetFont();
void FontFamily();
void InsertSpecialChar();
void TextRectStart();
void TextRectEnd();
void InsertFigure();
void InsertInlineFigure();
void SetFrameType();
void FontWeight();
void ChangeFontWeight();
void FontAngle();
void ChangeFontAngle();
void FontUnderline();
void ChangeFontUnderline();
void FontSubscript();
void ChangeFontSubscript();
void FontSuperscript();
void ChangeFontSuperscript();
void TextFlowStart();
void TextFlowTag();
void TextFlowEnd();
void InlineFrameStart();
void InlineFrameEnd();
void TableHeadStart();
void TableHeadEnd();
void TableBodyStart();
void TableBodyEnd();
void TableFootStart();
void TableFootEnd();
void TableRowStart();
void TableRowEnd();
void TableCellStart();
void TableCellColumns();
void TableCellEnd();
void NewCellLine();
int DoesTitleExist();
void HandleParaLine();
void MergeMarkerWithString();
void Finish();

#endif

extern int font_bold;
extern int font_italic;
extern int font_underline;
extern int font_subscript;
extern int font_superscript;
extern int in_code;

extern int in_table_title;
extern int in_table_body;
extern int table_count;
extern char *table_filenames[1000];
extern char table_filename[300];
extern struct FMCell *current_cell;
extern struct FMTable *current_table;

#endif /* TRANS_H */
